/*---------------------------------------------------------------------------------

---------------------------------------------------------------------------------*/
#include <nds.h>
#include <nds/arm9/console.h> //basic print funcionality
#include <stdio.h>
#include <fat.h>
#include <sys/stat.h>

#include <nds/arm9/video.h>
#include <string.h>
#include <unistd.h>

#include "nds_loader_arm9.h"
#include "file_browse.h"

//#define SELECT_DOCUMENT	// Choose a document to open instead of an NDS file

#ifndef SELECT_DOCUMENT
	const char DEFAULT_FILE[] = "fat0:/_BOOT_DS.NDS";

#else
	const char HANDLER_NAME[] = "Tuna-viDS";

	const char* HANDLER_LOCATIONS [] = {
		"fat0:/DS/tuna-viDS/tuna-viDS.nds",
		"fat0:/tuna-viDS.nds",
		"fat0:/data/tuna-viDS/tuna-viDS.nds",
	};

	const char DOCUMENT_EXT[] = ".avi";
#endif

void stop (void) 
{
	while (1) {
		swiWaitForVBlank();
	}
}

//---------------------------------------------------------------------------------
int main(void) {
//---------------------------------------------------------------------------------
	defaultExceptionHandler();

	struct stat st;

	std::string filename;
	const char* argv[2];
	char filePath[MAXPATHLEN * 2];
	int pathLen;

	// Enable VBlank for Vblank Interrupt Wait
	irqInit();
	irqEnable(IRQ_VBLANK);

	videoSetMode(0);	//not using the main screen

	// Subscreen as a console
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	vramSetBankH(VRAM_H_SUB_BG);
	SUB_BG0_CR = BG_MAP_BASE(15);
	BG_PALETTE_SUB[0]=0;   
	BG_PALETTE_SUB[255]=0xffff;
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(15), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

	iprintf ("Init'ing FAT...");
	if (fatInitDefault()) {
		iprintf ("okay\n");
	} else {
		iprintf ("fail!\n");
		stop();
	} 
	
#ifndef SELECT_DOCUMENT

	if (stat (DEFAULT_FILE, &st) >= 0) {
		filename = DEFAULT_FILE;
	} else {
		filename = browseForFile (".nds");
	}
	
	// Construct a command line if we weren't supplied with one
	getcwd (filePath, MAXPATHLEN);
	pathLen = strlen (filePath);
	strcpy (filePath + pathLen, filename.c_str());
	argv[0] = filePath;
	
	iprintf ("Running %s\n", argv[0]);
	runNdsFile (argv[0], 1, argv);

#else

	filename = browseForFile (DOCUMENT_EXT);
	
	argv[0] = NULL;
	for (size_t i = 0; i < sizeof(HANDLER_LOCATIONS) / sizeof(*HANDLER_LOCATIONS); i++) {
		if (stat (HANDLER_LOCATIONS[i], &st) >= 0) {
			argv[0] = HANDLER_LOCATIONS[i];
		}
	}

	if (argv[0] == NULL) {
		iprintf ("%s not found!\n", HANDLER_NAME);
		stop();
	}

	// Construct a command line if we weren't supplied with one
	getcwd (filePath, MAXPATHLEN);
	pathLen = strlen (filePath);
	strcpy (filePath + pathLen, filename.c_str());
	argv[1] = filePath;
	
	iprintf ("Opening %s\nwith %s\n", argv[1], argv[0]);
	runNdsFile (argv[0], 2, argv);

#endif

	iprintf ("Start failed\n");
	
	stop();

	return 0;
}
